
!****************************************************************************
!
!  PROGRAM: KSpread
!
!  PURPOSE:  Model the Neolithic expansion through Europe and with it the
!            spread of the mtDNA haplogroup K.
!
!  AUTHORS:   Neus Isern & Victor Lopez
!
!****************************************************************************


    program KSpread

    use GlobalVariables
    use ErrorHandling
    use TypeClassification
    use LandTravel, only: LandSeaDestinations
    use TravelWeights, only: SetTravelWeights
    use Simulation, only: RunSimulation
    
    implicit none

    ! Variables
    
    integer :: file_err  
    integer :: dumbIntValue, dumbIndex  
    integer(4) :: mountainEffect, seaTravelMode
    integer(4) :: maxIterations
    real(8) :: reprRateNeo, dumbHGValue, dumbFNeoValue, dumbFPalValue
  
    maxHGPop = nint(0.064*50*50)  !maximum HG population per cell
    maxFPop = nint(1.28*50*50)    !maximum F population per cell. maxFPop = maxFNeoPop + maxFPalPop    
    threshold = 0.1*maxFPop      !population level at which we record the first arrival
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! INPUT OF PARAMETER AND SIMULATION OPTIONS
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    !All the setup values are listed in a SimulationParameters.inp file and the values will be read from there
    open (12, file="SimulationParameters.inp", status="old",iostat=file_err)

    call OpenFileError(file_err, "There has been an error while opening 'SimulationParameters.inp' file.")
    read(12, *)

    !Will we consider mountain effect? 0 = NO, 1 = YES
    read(12, *)
    read(12, *) mountainEffect
    if ((mountainEffect /= 0) .and. (mountainEffect /= 1)) call InputError("Unknown mountain effect option")

    !Is sea travel allowed? 0 = No sea travel, 1 = Sea travel in straight lines
    read(12, *)
    read(12, *) seaTravelMode
    if ((seaTravelMode /= 0) .and. (seaTravelMode /= 1)) call InputError("Unknown sea travel mode")

    !maxSeaTravel = MaxSeaTravel(inKm) / 50
    read(12, *)
    read(12, *) maxSeaTravel
    if (maxSeaTravel > 10) then
        call InputError("Maximum sea travel allowed is 500km")
        !Modifiable at subroutine LongDistanceCoastTravel (SeaTravel module)
        !and class definition
    endif
    
    !Maximum number of iterations (or generations)
    read(12, *)
    read (12, *) maxIterations

    !Reproduction rate for mesolithic populations:
    read(12,*)
    read (12, *) reprRateHG
    
    !Reproduction rate for neolithic populations: R = exp(a*T)
    read(12,*)
    read (12, *) reprRateNeo
    
    !Persistence
    read(12,*)
    read (12, *) persistence
        
    !cultural transmision
    read(12,*)
    read(12, *) interbreeding
    
    close(12)
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!! Upload the grid and save the nodes in the Nodes array   !!!
    !!! Create an matrix with all the nodes positions           !!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    !Open the grid file
    open(11, file="Grid.dat", status = 'old', iostat=file_err) !Sea and mountains above 1750m    
    call OpenFileError(file_err, "There was an error while opening 'Grid.dat'.")
    
    !The first row of the file are three integers indicating the total number of nodes, columns and rows in the grid
    read(11,*) numNodes, x_numNodes, y_numNodes
    allocate(Nodes(0:numNodes))
    allocate(nodesGrid(0:x_numNodes+1, 0:y_numNodes+1))  
    !Border:
    nodesGrid=0  !Initially they are all border nodes

    
    !Node initialization:
    !- node_id
    !- node_type: 0=sea, 1=land, 2=mountains, 3=coastline   
    do i_node = 1, numNodes
        read(11,*) Nodes(i_node)%node_id, Nodes(i_node)%node_type
        Nodes(i_node)%frontArrival = 500  !The Neolithic front has not arrived yet
        Nodes(i_node)%land_sea = 0        !Variable initializations
    enddo
    
    !List of the eight nearest neighbors of each node. The first four are the four nearest neighbors
    do i_node = 1, numNodes
        !We need all the neighbors for sea travel
        read(11,*) dumbIntValue, (Nodes(i_node)%node_neighbors(dumbIndex), dumbIndex = 1, 8)
    enddo
    
    ! Node location in the grid
    ! - save the info in the node
    ! - save the location of each node in the nodesGrid
    do i_node = 1, numNodes
        read(11,*) dumbIntValue, Nodes(i_node)%x, Nodes(i_node)%y
        nodesGrid(Nodes(i_node)%x, Nodes(i_node)%y) = Nodes(i_node)%node_id
    enddo
    
    close(11)
    
    !Assign the longitude an latitude to each node
    open(11, file="LongLat.dat", status="old", iostat=file_err)
    call OpenFileError(file_err, "There was an error while opening file 'LongLat.dat'.")
    do i_node = 1, numNodes
        read(11,*) dumbIntValue, Nodes(i_node)%longitude, Nodes(i_node)%latitude
    enddo
    close(11)
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!  Initialize border node: Nodes(0)                                  !!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    !Node 0 is a grid border node. Border node type will be 5
    Nodes(0)%node_type = 5
    Nodes(0)%HGPop = 0
    Nodes(0)%HGPop2 = 0
    Nodes(0)%FNeoPop = 0
    Nodes(0)%FNeoPop2 = 0
    Nodes(0)%FPalPop = 0
    Nodes(0)%FPalPop2 = 0
    Nodes(0)%frontArrival = 500
    Nodes(0)%land_sea = 0

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!  Identify habitable and coastal nodes and save their id an array    !!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    ! Identifies habitable and non-habitable nodes, and coastal nodes
    mountains: select case (mountainEffect)
        case (0)
            ! No mountain effect
            call MountainsHabitable   ! Habitability module
            ! Changes mountains into land and counts and saves the id of
            !   habitable and non-habitable nodes
            ! Counts and saves the id of coastal nodes
            
        case (1)
            ! There is mountain effect
            call MountainsNotHabitable  ! Habitability module
            ! Counts and saves the id of habitable  and non-habitable nodes
            ! Counts and saves the id of coastal nodes
            
        case default
            call InputError("Incorrect value for mountain effect.")
    
    end select mountains
        
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!  Initialize Neolithic density                                        !!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  
    open(18, file="initial_dens.dat", status="old", iostat=file_err)
    call OpenFileError(file_err, "There was an error while opening 'initial_dens.dat' file.")    
    read(18,*)
    read(18,*) !The first two lines are is a comments
    
    do i_node = 1, numNodes
        read(18, *) dumbIntValue, dumbHGValue, dumbFNeoValue, dumbFPalValue
        Nodes(i_node)%HGPop = nint(dumbHGValue*maxHGPop)
        Nodes(i_node)%HGPop2 = 0
        Nodes(i_node)%FNeoPop = nint(dumbFNeoValue*maxFPop)
        Nodes(i_node)%FNeoPop2 = 0
        Nodes(i_node)%FPalPop = nint(dumbFPalValue*maxFPop) !En principi no hi ha FPal a t=0, per tant no afecta
        Nodes(i_node)%FPalPop2 = 0
        if ((Nodes(i_node)%FNeoPop + Nodes(i_node)%FPalPop) > threshold) Nodes(i_node)%frontArrival = 0
    enddo
    close(18)            

    
    !Make sure that non-habitable nodes have nul density
    do dumbIndex = 1, numNonHabitableNodes
       Nodes(nonHabitableNodes(dumbIndex))%HGPop = 0.
       Nodes(nonHabitableNodes(dumbIndex))%FNeoPop = 0.
       Nodes(nonHabitableNodes(dumbIndex))%FPalPop = 0.
    enddo   
    
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!! Checks which neighbors are suitable land destinations              !!!
    !!!   and which are sea neighbors that might be "used" for sea travel  !!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   
    call LandSeaDestinations
        
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!  Sets the travel destinations according to the seatravel mode    !!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        
    call SetTravelWeights(seaTravelMode)
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!  Simulation                                                      !!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    call RunSimulation(maxIterations, reprRateNeo) 
    
    deallocate(Nodes, coastNodes)
    deallocate(nodesGrid)
    end program KSpread